@@ -23,9 +23,7 @@
 #include "qresult.h"
 #include "pgapifunc.h"
 
-
-
-RETCODE
+static RETCODE
 set_statement_option(ConnectionClass *conn,
 					 StatementClass *stmt,
 					 SQLUSMALLINT fOption,
@@ -38,7 +36,7 @@ set_statement_option(ConnectionClass *conn,
 
 	if (conn)
 		ci = &(conn->connInfo);
-	else if (stmt)
+	else
 		ci = &(SC_get_conn(stmt)->connInfo);
 	switch (fOption)
 	{
@@ -217,9 +215,7 @@ set_statement_option(ConnectionClass *conn,
 		case SQL_USE_BOOKMARKS:
 			if (stmt)
 			{
-#if (ODBCVER >= 0x0300)
 				mylog("USE_BOOKMARKS %s\n", (vParam == SQL_UB_OFF) ? "off" : ((vParam == SQL_UB_VARIABLE) ? "variable" : "fixed"));
-#endif /* ODBCVER */
 				setval = vParam;
 				stmt->options.use_bookmarks = (SQLUINTEGER) setval;
 			}
@@ -245,8 +241,9 @@ set_statement_option(ConnectionClass *conn,
 				if (0 != vParam)
 					changed = TRUE;
 				break;
-#endif /* NOT_USED */
+#else
 				SC_set_error(stmt, STMT_OPTION_NOT_FOR_THE_DRIVER, "The option may be for MS SQL Server(Set)", func);
+#endif /* NOT_USED */
 			}
 			else if (conn)
 			{
@@ -299,7 +296,6 @@ PGAPI_SetConnectOption(HDBC hdbc,
 {
 	CSTR func = "PGAPI_SetConnectOption";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
-	ConnInfo   *ci = &(conn->connInfo);
 	char		changed = FALSE;
 	RETCODE		retval;
 	BOOL		autocomm_on;
@@ -330,19 +326,6 @@ PGAPI_SetConnectOption(HDBC hdbc,
 		case SQL_ROWSET_SIZE:
 		case SQL_SIMULATE_CURSOR:
 		case SQL_USE_BOOKMARKS:
-
-#if (ODBCVER < 0x0300)
-			{
-				int	i;
-				/* Affect all current Statements */
-				for (i = 0; i < conn->num_stmts; i++)
-				{
-					if (conn->stmts[i])
-						set_statement_option(NULL, conn->stmts[i], fOption, vParam);
-				}
-			}
-#endif /* ODBCVER */
-
 			/*
 			 * Become the default for all future statements on this
 			 * connection
@@ -376,11 +359,11 @@ PGAPI_SetConnectOption(HDBC hdbc,
 					CC_set_error(conn, CONN_INVALID_ARGUMENT_NO, "Illegal parameter value for SQL_AUTOCOMMIT", func);
 					return SQL_ERROR;
 			}
-			if (autocomm_on && SQL_AUTOCOMMIT_OFF != ci->autocommit_public)
+			if (autocomm_on && SQL_AUTOCOMMIT_OFF != conn->autocommit_public)
 				break;
-			else if (!autocomm_on && SQL_AUTOCOMMIT_OFF == ci->autocommit_public)
+			else if (!autocomm_on && SQL_AUTOCOMMIT_OFF == conn->autocommit_public)
 				break;
-			ci->autocommit_public = (autocomm_on ? SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF);
+			conn->autocommit_public = (autocomm_on ? SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF);
 			mylog("%s: AUTOCOMMIT: transact_status=%d, vparam=" FORMAT_LEN "\n", func, conn->transact_status, vParam);
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
@@ -413,17 +396,12 @@ PGAPI_SetConnectOption(HDBC hdbc,
 			switch (vParam)
 			{
 				case SQL_TXN_SERIALIZABLE:
-					if (PG_VERSION_GE(conn, 6.5) &&
-					    PG_VERSION_LE(conn, 7.0))
-						retval = SQL_ERROR;
 					break;
 				case SQL_TXN_REPEATABLE_READ:
 					if (PG_VERSION_LT(conn, 8.0))
 						retval = SQL_ERROR;
 					break;
 				case SQL_TXN_READ_COMMITTED:
-					if (PG_VERSION_LT(conn, 6.5))
-						retval = SQL_ERROR;
 					break;
 				case SQL_TXN_READ_UNCOMMITTED:
 					if (PG_VERSION_LT(conn, 8.0))
@@ -544,7 +522,6 @@ PGAPI_GetConnectOption(HDBC hdbc,
 {
 	CSTR func = "PGAPI_GetConnectOption";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
-	ConnInfo   *ci = &(conn->connInfo);
 	const char	*p = NULL;
 	SQLLEN		len = sizeof(SQLINTEGER);
 	SQLRETURN	result = SQL_SUCCESS;
@@ -564,7 +541,7 @@ PGAPI_GetConnectOption(HDBC hdbc,
 			break;
 
 		case SQL_AUTOCOMMIT:
-			*((SQLUINTEGER *) pvParam) = ci->autocommit_public;
+			*((SQLUINTEGER *) pvParam) = conn->autocommit_public;
 			break;
 
 		case SQL_CURRENT_QUALIFIER:		/* don't use qualifiers */
@@ -577,7 +554,7 @@ PGAPI_GetConnectOption(HDBC hdbc,
 			break;
 
 		case SQL_PACKET_SIZE:	/* NOT SUPPORTED */
-			*((SQLUINTEGER *) pvParam) = ci->drivers.socket_buffersize;
+			*((SQLUINTEGER *) pvParam) = 4096;
 			break;
 
 		case SQL_QUIET_MODE:	/* NOT SUPPORTED */
@@ -598,12 +575,10 @@ PGAPI_GetConnectOption(HDBC hdbc,
 			mylog(" val=%d\n", *((SQLUINTEGER *) pvParam));
                         break;
 
-#if (ODBCVER >= 0x0351)
 		case SQL_ATTR_ANSI_APP:
 			*((SQLUINTEGER *) pvParam) = CC_is_in_ansi_app(conn);
 			mylog("ANSI_APP val=%d\n", *((SQLUINTEGER *) pvParam));
                         break;
-#endif /* ODBCVER */
 
 			/* These options should be handled by driver manager */
 		case SQL_ODBC_CURSORS:
@@ -698,6 +673,7 @@ PGAPI_GetStmtOption(HSTMT hstmt,
 	QResultClass *res;
 	SQLLEN		ridx;
 	SQLINTEGER	len = sizeof(SQLINTEGER);
+	UInt4		bookmark;
 
 	mylog("%s: entering...\n", func);
 
@@ -750,7 +726,8 @@ PGAPI_GetStmtOption(HSTMT hstmt,
 				return SQL_ERROR;
 			}
 
-			*((SQLULEN *) pvParam) = SC_get_bookmark(stmt);
+			bookmark = (UInt4) SC_get_bookmark(stmt);
+			memcpy(pvParam, &bookmark, sizeof(UInt4));
 
 			break;
 
