@@ -215,6 +215,7 @@ QR_Constructor(void)
 		rv->aborted = FALSE;
 
 		rv->cache_size = 0;
+		rv->cmd_fetch_size = 0;
 		rv->rowset_size_include_ommitted = 1;
 		rv->move_direction = 0;
 		rv->keyset = NULL;
@@ -579,7 +580,7 @@ BOOL
 QR_from_PGresult(QResultClass *self, StatementClass *stmt, ConnectionClass *conn, const char *cursor, PGresult **pgres)
 {
 	CSTR func = "QR_from_PGResult";
-	int			num_io_params;
+	int			num_io_params, num_cached_rows;
 	int			i;
 	Int2		paramType;
 	IPDFields  *ipdopts;
@@ -591,9 +592,11 @@ QR_from_PGresult(QResultClass *self, StatementClass *stmt, ConnectionClass *conn
 	char	   *new_field_name;
 	Int2		dummy1, dummy2;
 	int			cidx;
+	BOOL		reached_eof_now = FALSE;
 
-	/* First, get column information */
-	QR_set_conn(self, conn);
+	if (NULL != conn)
+		/* First, get column information */
+		QR_set_conn(self, conn);
 
 	/* at first read in the number of fields that are in the query */
 	new_num_fields = PQnfields(*pgres);
@@ -636,7 +639,7 @@ QR_from_PGresult(QResultClass *self, StatementClass *stmt, ConnectionClass *conn
 		self->num_fields = CI_get_num_fields(QR_get_fields(self));
 		if (QR_haskeyset(self))
 			self->num_fields -= self->num_key_fields;
-		if (stmt)
+		if (stmt && conn)
 		{
 			num_io_params = CountParameters(stmt, NULL, &dummy1, &dummy2);
 			if (stmt->proc_return > 0 ||
@@ -663,17 +666,30 @@ inolog("!![%d].PGType %u->%u\n", i, PIC_get_pgtype(ipdopts->parameters[i]), CI_g
 
 
 	/* Then, get the data itself */
+	num_cached_rows = self->num_cached_rows;
 	if (!QR_read_tuples_from_pgres(self, pgres))
 		return FALSE;
 
 inolog("!!%p->cursTup=%d total_read=%d\n", self, self->cursTuple, self->num_total_read);
 	if (!QR_once_reached_eof(self) && self->cursTuple >= (Int4) self->num_total_read)
 		self->num_total_read = self->cursTuple + 1;
+	/* EOF is 'fetched < fetch requested' */
+	if (self->num_cached_rows - num_cached_rows < self->cmd_fetch_size)
+	{
+		mylog("detect EOF %d - %d < %d\n", self->num_cached_rows, num_cached_rows, self->cmd_fetch_size);
+		reached_eof_now = TRUE;
+		QR_set_reached_eof(self);
+	}
+	if (reached_eof_now && self->cursTuple < (Int4) self->num_total_read)
+		self->cursTuple = self->num_total_read;
 
-	/* Force a read to occur in next_tuple */
-	QR_set_next_in_cache(self, 0);
-	QR_set_rowstart_in_cache(self, 0);
-	self->key_base = 0;
+	if (NULL != conn)
+	{
+		/* Force a read to occur in next_tuple */
+		QR_set_next_in_cache(self, 0);
+		QR_set_rowstart_in_cache(self, 0);
+		self->key_base = 0;
+	}
 
 	/*
 	 * Also fill in command tag. (Typically, it's SELECT, but can also be
@@ -1165,6 +1181,7 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 
 	/* don't read ahead for the next tuple (self) ! */
 	qi.row_size = self->cache_size;
+	qi.fetch_size = fetch_size;
 	qi.result_in = self;
 	qi.cursor = NULL;
 	res = CC_send_query(conn, fetch, &qi, 0, stmt);
@@ -1182,10 +1199,7 @@ inolog("clear obsolete %d tuples\n", num_backend_rows);
 	self->tupleField = NULL;
 	num_rows_in = self->num_cached_rows;
 
-	if (curr_eof)
-		reached_eof_now = curr_eof;
-	else
-		curr_eof = reached_eof_now = (QR_once_reached_eof(self) && self->cursTuple >= (Int4)self->num_total_read);
+	curr_eof = reached_eof_now = (QR_once_reached_eof(self) && self->cursTuple >= (Int4)self->num_total_read);
 inolog("reached_eof_now=%d\n", reached_eof_now);
 
 	mylog("_%s: PGresult: fetch_total = %d & this_fetch = %d\n", func, self->num_total_read, self->num_cached_rows);
@@ -1202,12 +1216,6 @@ inolog("reached_eof_now=%d\n", reached_eof_now);
 		{
 			mylog("%s: reached eof now\n", func);
 			QR_set_reached_eof(self);
-			if (self->cursTuple >= (Int4) self->num_total_read)
-			{
-				self->num_total_read = self->cursTuple + 1;
-inolog("mayumi setting total_read to %d\n", self->num_total_read);
-			}
-			self->cursTuple++;
 			if (self->ad_count > 0 && cur_fetch < fetch_size)
 			{
 				/* We have to append the tuples(keys) info from the added tuples(keys) here */
